using Styx;
using Styx.Common;
using Styx.WoWInternals;
using Styx.CommonBot;
using Styx.CommonBot.Profiles;
using Styx.CommonBot.Routines;
using Styx.TreeSharp;
using Action = Styx.TreeSharp.Action;
using System;
using System.Windows.Forms;
using System.Windows.Media;

namespace Astoria
{
    public class Astoria_Bot : BotBase
    {
        private Composite _root;

        public override string Name
        {
            get
            {
                return "Astoria Bot";
            }
        }
        public override Composite Root
        {
            get
            {
                return _root ?? (_root = RootBehavior);
            }
        }

        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All & ~(PulseFlags.Looting | PulseFlags.CharacterManager); }
        }

        public bool IsPaused { get; set; }

        public override void Start()
        {
            TreeRoot.TicksPerSecond = 30;
            ProfileManager.LoadEmpty();
            Logging.Write("Astoria Bot Enable - 30 ticks Framework - Shift + Q for pause/resume.");

            HotkeysManager.Register("Astoria Bot Pause",
                Keys.Q,
                ModifierKeys.Shift,
                hk =>
                {
                    IsPaused = !IsPaused;
                    if (IsPaused)
                    {
                        AddToast("Paused", Colors.Red);
                        TreeRoot.TicksPerSecond = 5;
                        Logging.Write("Astoria Bot is paused.");
                    }
                    else
                    {
                        AddToast("Resumed", Colors.Green);
                        TreeRoot.TicksPerSecond = 30;
                        Logging.Write("Astoria Bot is resumed.");
                    }
                });
        }

        private void AddToast(string text, Color color)
        {
            StyxWoW.Overlay.AddToast(() => text, TimeSpan.FromSeconds(2), color, Colors.Black, new FontFamily("Courier"));
        }

        private Composite RootBehavior
        {
            get
            {
                return
                    new PrioritySelector(
                        new Decorator(ret => IsPaused,
                        new Action(ret => RunStatus.Success)),
                    new Switch<bool>(ret => StyxWoW.Me.Combat,
                    new SwitchArgument<bool>(true, RoutineManager.Current.CombatBehavior),
                    new SwitchArgument<bool>(false, RoutineManager.Current.PreCombatBuffBehavior)
                ));
            }
        }
    }
}